@@ -11,7 +11,7 @@
 
 extern struct gps_type_t zodiac_binary;
 
-#if defined(NMEA_ENABLE) || defined(SIRFII_ENABLL) || defined(EVERMORE_ENABLE)  || defined(ITALK_ENABLE) 
+#if defined(NMEA_ENABLE) || defined(SIRFII_ENABLE) || defined(EVERMORE_ENABLE)  || defined(ITALK_ENABLE) 
 ssize_t pass_rtcm(struct gps_device_t *session, char *buf, size_t rtcmbytes)
 /* most GPSes take their RTCM corrections straight up */
 {
@@ -65,9 +65,13 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 	    gpsd_report(1, "unknown sentence: \"%s\"\n", session->outbuffer);
 	}
 #ifdef NTPSHM_ENABLE
-	if ((st & TIME_SET) != 0)
-	    /* this magic number is derived from observation */
-	    (void)ntpshm_put(session, session->gpsdata.fix.time + 0.675);
+	/* this magic number is derived from observation */
+     if ((st & TIME_SET) != 0 &&
+         (session->gpsdata.fix.time != session->driver.nmea.last_fixtime)) {
+             /* this magic number is derived from observation */
+             (void)ntpshm_put(session, session->gpsdata.fix.time + 0.675);
+             session->driver.nmea.last_fixtime = session->gpsdata.fix.time;
+     }
 #endif /* NTPSHM_ENABLE */
 	return st;
     } else
