@@ -23,6 +23,7 @@ static double bilinear(double x1, double y1, double x2, double y2, double x, dou
 
 
 /* return geoid separtion (MSL - WGS84) in meters, given a lat/lot in degrees */
+/*@ +charint @*/
 double wgs84_separation(double lat, double lon)
 {
 #define GEOID_ROW	19
@@ -47,6 +48,7 @@ double wgs84_separation(double lat, double lon)
 	/* 70N */   2,  2,  1, -1, -3, -7,-14,-24,-27,-25,-19,  3, 24, 37, 47, 60, 61, 58, 51, 43, 29, 20, 12,  5, -2,-10, -14,-12,-10,-14,-12, -6, -2,  3,  6,  4,  2,
 	/* 80N */   3,  1, -2, -3, -3, -3, -1,  3,  1,  5,  9, 11, 19, 27, 31, 34, 33, 34, 33, 34, 28, 23, 17, 13,  9,  4,   4,  1, -2, -2,  0,  2,  3,  2,  1,  1,  3,
 	/* 90N */  13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13,  13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13}; 
+/*@ -charint @*/
     int	ilat, ilon;
     int	ilat1, ilat2, ilon1, ilon2;
 
@@ -62,13 +64,15 @@ double wgs84_separation(double lat, double lon)
 	ilon1*10.-180.,ilat1*10.-90.,
 	ilon2*10.-180.,ilat2*10.-90.,
 	lon,           lat,
-	geoid_delta[ilon1+ilat1*GEOID_COL], geoid_delta[ilon2+ilat1*GEOID_COL],
-	geoid_delta[ilon1+ilat2*GEOID_COL], geoid_delta[ilon2+ilat2*GEOID_COL]
+	(double)geoid_delta[ilon1+ilat1*GEOID_COL], 
+	(double)geoid_delta[ilon2+ilat1*GEOID_COL],
+	(double)geoid_delta[ilon1+ilat2*GEOID_COL], 
+	(double)geoid_delta[ilon2+ilat2*GEOID_COL]
 	);
 }
 
 
-void ecef_to_wgs84fix(struct gps_fix_t *fix,
+void ecef_to_wgs84fix(struct gps_data_t *gpsdata,
 			double x, double y, double z, 
 			double vx, double vy, double vz)
 /* fill in WGS84 position/velocity fields from ECEF coordinates */
@@ -82,23 +86,26 @@ void ecef_to_wgs84fix(struct gps_fix_t *fix,
 
     /* geodetic location */
     lambda = atan2(y,x);
-    p = sqrt(pow(x,2) + pow(y,2));
+    /*@ -evalorder @*/ 
+    p = sqrt(pow(x,2) + pow(y,2)); 
     theta = atan2(z*a,p*b);
     phi = atan2(z + e_2*b*pow(sin(theta),3),p - e2*a*pow(cos(theta),3));
     n = a / sqrt(1.0 - e2*pow(sin(phi),2));
     h = p / cos(phi) - n;
-    fix->latitude = phi * RAD_2_DEG;
-    fix->longitude = lambda * RAD_2_DEG;
-    fix->altitude = h - wgs84_separation(fix->latitude, fix->longitude);
+    gpsdata->newdata.latitude = phi * RAD_2_DEG;
+    gpsdata->newdata.longitude = lambda * RAD_2_DEG;
+    gpsdata->separation = wgs84_separation(gpsdata->newdata.latitude, gpsdata->newdata.longitude);
+    gpsdata->newdata.altitude = h - gpsdata->separation;
     /* velocity computation */
     vnorth = -vx*sin(phi)*cos(lambda)-vy*sin(phi)*sin(lambda)+vz*cos(phi);
     veast = -vx*sin(lambda)+vy*cos(lambda);
-    fix->climb = vx*cos(phi)*cos(lambda)+vy*cos(phi)*sin(lambda)+vz*sin(phi);
-    fix->speed = RAD_2_DEG * sqrt(pow(vnorth,2) + pow(veast,2));
+    gpsdata->newdata.climb = vx*cos(phi)*cos(lambda)+vy*cos(phi)*sin(lambda)+vz*sin(phi);
+    gpsdata->newdata.speed = sqrt(pow(vnorth,2) + pow(veast,2));
     heading = atan2(veast,vnorth);
+    /*@ +evalorder @*/
     if (heading < 0)
 	heading += 2 * PI;
-    fix->track = heading * RAD_2_DEG;
+    gpsdata->newdata.track = heading * RAD_2_DEG;
 }
 
 #ifdef TESTMAIN
