@@ -8,9 +8,16 @@ from math import *
 # Needed in all versions of Python that don't implement
 # PEP 75 (http://python.fyxm.net/peps/pep-0754.html).
 # This includes at least versions up to 2.3.4.
-PosInf = 1e300000
-NaN = PosInf/PosInf
-def isnan(x): return x == NaN
+# PosInf = 1e300000
+# NaN = PosInf/PosInf
+# IEEE754 says NaN == NaN is always false!
+# so this is wrong: 
+#    def isnan(x): return x == NaN
+
+# so let's use PosInf as a proxy
+# this is not mathmetically correct but good enough for altitude
+NaN = 1e10
+def isnan(x): return x > 1e9
 
 ONLINE_SET	= 0x00000001
 TIME_SET	= 0x00000002
@@ -148,6 +155,7 @@ class gpsdata:
 	self.baudrate = 0
 	self.stopbits = 0
 	self.cycle = 0
+	self.mincycle = 0
 	self.device = None
 	self.devices = []
 
@@ -273,8 +281,10 @@ class gps(gpsdata):
 	      if data == '?':
 		  self.cycle = -1
 		  self.device = None
-	      else:
-		  self.cycle = int(data)
+	      elif len(data.split()) == 2:
+		  (self.cycle, self.mincycle) = map(float, data)
+              else:
+		  self.mincycle = self.cycle = float(data)
 	    elif cmd in ('D', 'd'):
 	      self.utc = data
 	      self.gps_time = isotime(self.utc)
@@ -324,10 +334,6 @@ class gps(gpsdata):
 		    self.fix.latitude = default(3)
 		    self.fix.longitude = default(4)
 		    self.fix.altitude = default(5)
-		    if not isnan(self.fix.altitude):
-			self.fix.mode = MODE_2D
-		    else:
-			self.fix.mode = MODE_3D
 		    self.fix.eph = default(6)
 		    self.fix.epv = default(7)
 		    self.fix.track = default(8)
@@ -336,6 +342,13 @@ class gps(gpsdata):
 		    self.fix.epd = default(11)
 		    self.fix.eps = default(12)
 		    self.fix.epc = default(13)
+                    if len(fields) > 14:
+                        self.fix.mode = default(14)
+                    else:
+                        if not isnan(self.fix.altitude):
+                            self.fix.mode = MODE_2D
+                        else:
+                            self.fix.mode = MODE_3D
 		    self.valid |= TIME_SET|TIMERR_SET|LATLON_SET|MODE_SET
 		    if self.fix.mode == MODE_3D:
 			self.valid |= ALTITUDE_SET | CLIMB_SET
