@@ -60,10 +60,6 @@
 
 #define QLEN			5
 
-/* Where to find the list of DGPS correction servers, if there is one */
-#define DGPS_SERVER_LIST	"/usr/share/gpsd/dgpsip-servers"
-
-
 /*
  * The name of a tty device from which to pick up whatever the local
  * owning group for tty devices is.  Used when we drop privileges.
@@ -78,8 +74,10 @@ static jmp_buf restartbuf;
 static struct gps_context_t context = {
     .valid        = 0, 
     .sentdgps     = false, 
+    .dgnss_service  = dgnss_none,
     .fixcnt       = 0, 
     .dsock        = -1, 
+    .dgnss_privdata = NULL,
     .rtcmbytes    = 0, 
     .rtcmbuf      = {'\0'}, 
     .rtcmtime     = 0,
@@ -142,7 +140,9 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 	va_list ap;
 
 #if defined(PPS_ENABLE)
+	/*@ -unrecog  (splint has no pthread declarations as yet) @*/
 	(void)pthread_mutex_lock(&report_mutex);
+	/* +unrecog */
 #endif /* PPS_ENABLE */
 	(void)strcpy(buf, "gpsd: ");
 	va_start(ap, fmt) ;
@@ -161,21 +161,27 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 	else
 	    (void)fputs(buf2, stderr);
 #if defined(PPS_ENABLE)
+	/*@ -unrecog (splint has no pthread declarations as yet) @*/
 	(void)pthread_mutex_unlock(&report_mutex);
+	/* +unrecog */
 #endif /* PPS_ENABLE */
     }
 }
 
 static void usage(void)
 {
-    (void)printf("usage: gpsd [-d dgpsip-server] [-D n] [-F sockfile] [-P pidfile] [-S port] [-h] device...\n\
+    (void)printf("usage: gpsd [-n] [-N] [-d dgnss-service] [-D n] [-F sockfile] [-P pidfile] [-S port] [-h] device...\n\
   Options include: \n\
-  -d host[:port]         	= set DGPS server \n\
-  -F sockfile                   = specift control socket location\n\
+  -n                            = don't wait for client connects to poll GPS\n\
+  -N                            = don't go into background\n\
+  -d [{dgpsip|ntrip}://][user:passwd@]host[:port][/stream] = set DGPS service\n\
+  -F sockfile                   = specify control socket location\n\
   -P pidfile              	= set file to record process ID \n\
   -D integer (default 0)  	= set debug level \n\
   -S integer (default %s)	= set port for daemon \n\
-  -h                     	= help message \n",
+  -h                     	= help message \n\
+  -V                            = emit version and exit.\n",
+
 	   DEFAULT_GPSD_PORT);
 }
 
@@ -198,7 +204,7 @@ static bool have_fix(struct gps_device_t *device)
     }
     VALIDATION_COMPLAINT(3, "GPS has no fix");
     return false;
-#undef VALIDATION_CONSTRAINT
+#undef VALIDATION_COMPLAINT
 }
 
 static int passivesock(char *service, char *protocol, int qlen)
@@ -237,6 +243,9 @@ static int passivesock(char *service, char *protocol, int qlen)
     }
     if (bind(s, (struct sockaddr *) &sin, (int)sizeof(sin)) < 0) {
 	gpsd_report(0, "Can't bind to port %s\n", service);
+        if (errno == EADDRINUSE) {
+                gpsd_report(0, "Maybe gpsd is already running!\n");
+        }
 	return -1;
     }
     if (type == SOCK_STREAM && listen(s, qlen) < 0) {
@@ -259,8 +268,7 @@ static int filesock(char *filename)
     }
     (void)strcpy(addr.sun_path, filename);
     /*@i1@*/addr.sun_family = AF_UNIX;
-    (void)bind(sock, (struct sockaddr *) &addr, 
-	 (int)(strlen(addr.sun_path) + sizeof (addr.sun_family)));
+    (void)bind(sock, (struct sockaddr *) &addr,  (int)sizeof(addr));
     if (listen(sock, QLEN) < 0) {
 	gpsd_report(0, "can't listen on local socket %s\n", filename);
 	return -1;
@@ -283,7 +291,7 @@ static struct gps_device_t channels[MAXDEVICES];
 #define syncing(chp)	(chp->gpsdata.gps_fd>-1&& chp->packet_type==BAD_PACKET)
 
 static struct subscriber_t {
-    double active;			/* when subscriber signed on */
+    double active;			/* when subscriber last polled for data */
     bool tied;				/* client set device with F */
     bool watcher;			/* is client in watcher mode? */
     int raw;				/* is client in raw mode? */
@@ -422,7 +430,6 @@ static bool allocation_policy(struct gps_device_t *channel,
 	return true;
     else
 	return false;
-    return true;
 }
 
 /*@ -branchstate -usedef -globstate @*/
@@ -454,7 +461,9 @@ static bool assign_channel(struct subscriber_t *user)
 	gpsd_report(1,"client(%d): channel %d already active.\n",
 		    user-subscribers, user->device->gpsdata.gps_fd);
     else {
+#ifndef FIXED_PORT_SPEED
 	gpsd_deactivate(user->device);
+#endif
 	if (gpsd_activate(user->device) < 0) {
 	    
 	    gpsd_report(1, "client(%d): channel activation failed.\n", user-subscribers);
@@ -478,7 +487,7 @@ static bool assign_channel(struct subscriber_t *user)
 /*@ +branchstate +usedef +globstate @*/
 
 #ifdef RTCM104_SERVICE
-static int handle_dgpsip_request(int cfd UNUSED, char *buf UNUSED, int buflen UNUSED)
+static int handle_rtcm_request(int cfd UNUSED, char *buf UNUSED, int buflen UNUSED)
 /* interpret a client request; cfd is the socket back to the client */
 {
     return 0;	/* not actually interpreting these yet */
@@ -538,6 +547,7 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 	    else
 		(void)strcpy(phrase, ",A=?");
 	    break;
+#ifndef FIXED_PORT_SPEED
 	case 'B':		/* change baud rate (SiRF/Zodiac only) */
 	    if (assign_channel(whoami) && whoami->device->device_type!=NULL && *p=='=' && privileged_user(whoami)) {
 		i = atoi(++p);
@@ -580,6 +590,7 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 		(void)strcpy(phrase, ",B=?");
 	    }
 	    break;
+#endif
 	case 'C':
 	    if (!assign_channel(whoami) || whoami->device->device_type==NULL)
 		(void)strcpy(phrase, ",C=?");
@@ -653,6 +664,14 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 	    else
 		(void)snprintf(phrase, sizeof(phrase), ",G=GPS");
 	    break;
+#ifdef HEADING_FIX
+	case 'H':
+	    if (assign_channel(whoami) && have_fix(whoami->device) && isnan(whoami->device->gpsdata.fix.heading)==0)
+		(void)snprintf(phrase, sizeof(phrase), ",H=%.4f", whoami->device->gpsdata.fix.heading);
+	    else
+		(void)strcpy(phrase, ",H=?");
+	    break;
+#endif /* HEADING_FIX */
 	case 'I':
 	    if (assign_channel(whoami) && whoami->device->device_type!=NULL)
 		(void)snprintf(phrase, sizeof(phrase), ",I=%s", 
@@ -674,7 +693,7 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 	    phrase[strlen(phrase)-1] = '\0';
 	    break;
 	case 'L':
-	    (void)snprintf(phrase, sizeof(phrase), ",L=2 " VERSION " abcdefgiklmnopqrstuvwxyz");	//hj
+	    (void)snprintf(phrase, sizeof(phrase), ",L=3 " VERSION " abcdefgiklmnopqrstuvwxyz");	//hj
 	    break;
 	case 'M':
 	    if (!assign_channel(whoami) && (!whoami->device || whoami->device->gpsdata.fix.mode == MODE_NOT_SEEN))
@@ -789,6 +808,12 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 			     " %5.2f", whoami->device->gpsdata.fix.epc);		    
 		else
 		    (void)strcat(phrase, "        ?");
+		if (whoami->device->gpsdata.fix.mode > 0)
+		    (void)snprintf(phrase+strlen(phrase),
+			     sizeof(phrase)-strlen(phrase),
+			     " %d", whoami->device->gpsdata.fix.mode);		    
+		else
+		    (void)strcat(phrase, "        ?");
 	    }
 	    break;
 	case 'P':
@@ -865,8 +890,8 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 		(void)strcpy(phrase, ",U=?");
 	    break;
 	case 'V':
-	    if (assign_channel(whoami) && have_fix(whoami->device) && isnan(whoami->device->gpsdata.fix.track)==0)
-		(void)snprintf(phrase, sizeof(phrase), ",V=%.3f", whoami->device->gpsdata.fix.speed / KNOTS_TO_KPH);
+	    if (assign_channel(whoami) && have_fix(whoami->device) && isnan(whoami->device->gpsdata.fix.speed)==0)
+		(void)snprintf(phrase, sizeof(phrase), ",V=%.3f", whoami->device->gpsdata.fix.speed * MPS_TO_KNOTS);
 	    else
 		(void)strcpy(phrase, ",V=?");
 	    break;
@@ -963,8 +988,10 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 	    }
 	    break;
         case '$':
-	    if (whoami->device->gpsdata.sentence_time!=0)
-		(void)snprintf(phrase, sizeof(phrase), ",$=%s %d %f %f %f %f %f %f",
+	    if (!assign_channel(whoami))
+		(void)strcpy(phrase, ",$=?");
+	    else if (whoami->device->gpsdata.sentence_time!=0)
+		(void)snprintf(phrase, sizeof(phrase), ",$=%s %d %lf %lf %lf %lf %lf %lf",
 			whoami->device->gpsdata.tag,
 			(int)whoami->device->gpsdata.sentence_length,
 			whoami->device->gpsdata.sentence_time,
@@ -974,7 +1001,7 @@ static int handle_gpsd_request(int cfd, char *buf, int buflen)
 			whoami->device->poll_times[cfd] - whoami->device->gpsdata.sentence_time,
 			timestamp() - whoami->device->gpsdata.sentence_time);
 	    else
-		(void)snprintf(phrase, sizeof(phrase), ",$=%s %d 0 %f %f %f %f %f",
+		(void)snprintf(phrase, sizeof(phrase), ",$=%s %d 0 %lf %lf %lf %lf %lf",
 			whoami->device->gpsdata.tag,
 			(int)whoami->device->gpsdata.sentence_length,
 			whoami->device->gpsdata.d_xmit_time,
@@ -1057,11 +1084,11 @@ int main(int argc, char *argv[])
     static bool nowait = false;
     static int st, csock = -1;
     static gps_mask_t changed;
-    static char *dgpsserver = NULL;
+    static char *dgnss_service = NULL;
     static char *gpsd_service = NULL; 
 #ifdef RTCM104_SERVICE
     static char *rtcm_service = NULL; 
-    static int nsock, rsock = -1;
+    static int nsock;
 #endif /* RTCM104_SERVICE */
     static char *control_socket = NULL;
     struct gps_device_t *device, *channel;
@@ -1076,7 +1103,7 @@ int main(int argc, char *argv[])
 #endif /* RTCM104_ENABLE */
 
     debuglevel = 0;
-    while ((option = getopt(argc, argv, "F:D:S:d:fhNnpP:v"
+    while ((option = getopt(argc, argv, "F:D:S:d:fhNnpP:V"
 #ifdef RTCM104_SERVICE
 			    "R:"
 #endif /* RTCM104_SERVICE */
@@ -1100,7 +1127,7 @@ int main(int argc, char *argv[])
 	    gpsd_service = optarg;
 	    break;
 	case 'd':
-	    dgpsserver = optarg;
+	    dgnss_service = optarg;
 	    break;
 	case 'n':
 	    nowait = true;
@@ -1112,7 +1139,7 @@ int main(int argc, char *argv[])
 	case 'P':
 	    pid_file = optarg;
 	    break;
-	case 'v':
+	case 'V':
 	    (void)printf("gpsd %s\n", VERSION);
 	    exit(0);
 	case 'h': case '?':
@@ -1122,6 +1149,12 @@ int main(int argc, char *argv[])
 	}
     }
 
+#ifdef FIXED_PORT_SPEED
+    /* Asssume that if we're running with FIXED_PORT_SPEED we're some sort
+     * of embedded configuration where we don't want to wait for connect */
+    nowait = true;
+#endif
+
     if (!control_socket && optind >= argc) {
 	gpsd_report(0, "can't run with neither control socket nor devices\n");
 	exit(1);
@@ -1179,8 +1212,8 @@ int main(int argc, char *argv[])
     gpsd_report(1, "listening on port %s\n", rtcm_service);
 #endif /* RTCM104_SERVICE */
 
-    if (dgpsserver) {
-        int dsock = dgpsip_open(&context, dgpsserver);
+    if (dgnss_service) {
+	int dsock = dgnss_open(&context, dgnss_service);
 	if (dsock >= 0)
 	    FD_SET(dsock, &all_fds);
     }
@@ -1335,18 +1368,18 @@ int main(int argc, char *argv[])
 	    socklen_t alen = (socklen_t)sizeof(fsin);
 	    /*@i1@*/int ssock = accept(nsock, (struct sockaddr *)&fsin, &alen);
 
-	    if (rsock < 0)
+	    if (ssock < 0)
 		gpsd_report(0, "accept: %s\n", strerror(errno));
 	    else {
-		int opts = fcntl(rsock, F_GETFL);
+		int opts = fcntl(ssock, F_GETFL);
 
 		if (opts >= 0)
-		    (void)fcntl(rsock, F_SETFL, opts | O_NONBLOCK);
-		gpsd_report(3, "client connect on %d\n", rsock);
+		    (void)fcntl(ssock, F_SETFL, opts | O_NONBLOCK);
+		gpsd_report(3, "client connect on %d\n", ssock);
 		FD_SET(ssock, &all_fds);
-		subscribers[rsock].active = true;
-		subscribers[rsock].tied = false;
-		subscribers[rsock].requires = RTCM104;
+		subscribers[ssock].active = true;
+		subscribers[ssock].tied = false;
+		subscribers[ssock].requires = RTCM104;
 	    }
 	    FD_CLR(nsock, &rfds);
 	}
@@ -1367,10 +1400,10 @@ int main(int argc, char *argv[])
 	    FD_CLR(csock, &rfds);
 	}
 
-	/* be ready for DGPSIP reports */
-	if (context.dsock >= 0 && FD_ISSET(context.dsock, &rfds))
-	    dgpsip_poll(&context);
-
+	if (context.dsock >= 0 && FD_ISSET(context.dsock, &rfds)) {
+	    /* be ready for DGPS reports */
+	    dgnss_poll(&context);
+	}
 	/* read any commands that came in over control sockets */
 	for (cfd = 0; cfd < FD_SETSIZE; cfd++)
 	    if (FD_ISSET(cfd, &control_fds)) {
@@ -1390,9 +1423,9 @@ int main(int argc, char *argv[])
 	    if (!allocated_channel(channel))
 		continue;
 
-	    /* pass the current DGPSIP correction to the GPS if new */
+	    /* pass the current RTCM correction to the GPS if new */
 	    if (channel->device_type)
-		dgpsip_relay(channel);
+		rtcm_relay(channel);
 
 	    /* get data from the device */
 	    changed = 0;
@@ -1450,14 +1483,12 @@ int main(int argc, char *argv[])
 	}
 
 #ifdef NOT_FIXED
-	/* may be time to hunt up a DGPSIP server */
 	if (context.fixcnt > 0 && context.dsock == -1) {
 	    for (channel=channels; channel < channels+MAXDEVICES; channel++) {
 		if (channel->gpsdata.fix.mode > MODE_NO_FIX) {
-		    dgpsip_autoconnect(&context,
-				       channel->gpsdata.fix.latitude,
-				       channel->gpsdata.fix.longitude,
-				       DGPS_SERVER_LIST);
+		    dgnss_autoconnect(&context,
+				      channel->gpsdata.fix.latitude,
+				      channel->gpsdata.fix.longitude);
 		    break;
 		}
 	    }
@@ -1481,14 +1512,22 @@ int main(int argc, char *argv[])
 		    gpsd_report(1, "<= client(%d): %s", cfd, buf);
 
 #ifdef RTCM104_SERVICE
-		    if (subscribers[cfd].rtcm) {
-			if (handle_dgpsip_request(cfd, buf, buflen) < 0)
+		    if (subscribers[cfd].requires==RTCM104
+			|| subscribers[cfd].requires==ANY) {
+			if (handle_rtcm_request(cfd, buf, buflen) < 0)
 			    detach_client(cfd);
 		    } else
 #endif /* RTCM104_SERVICE */
 		    {
-			if (subscribers[cfd].device)
-			    subscribers[cfd].device->poll_times[cfd] = timestamp();
+			if (subscribers[cfd].device){
+		            /*
+			     * when a command comes in, to update .active to
+			     * timestamp() so we don't close the connection
+			     * after POLLER_TIMEOUT seconds. This makes
+			     * POLLER_TIMEOUT useful.
+			     */
+			    subscribers[cfd].active = subscribers[cfd].device->poll_times[cfd] = timestamp();
+                        }
 			if (handle_gpsd_request(cfd, buf, buflen) < 0)
 			    detach_client(cfd);
 		    }
