@@ -46,9 +46,11 @@ struct gps_context_t {
     int valid;				/* member validity flags */
 #define LEAP_SECOND_VALID	0x01	/* we have or don't need correction */
     /* DGPSIP status */
-    bool sentdgps;			/* have we sent a DGPSIP R report? */
+    bool sentdgps;			/* have we sent a DGPS report? */
+    enum { dgnss_none, dgnss_dgpsip, dgnss_ntrip } dgnss_service;	/* type of DGNSS service */
     int fixcnt;				/* count of good fixes seen */
-    int dsock;				/* socket to DGPS server */
+    int dsock;			        /* socket to DGPSIP server/Ntrip caster */
+    void *dgnss_privdata;		/* DGNSS service specific data */
     ssize_t rtcmbytes;			/* byte count of last RTCM104 report */
     char rtcmbuf[RTCM_MAX];		/* last RTCM104 report */
     double rtcmtime;			/* timestamp of last RTCM104 report */ 
@@ -160,6 +162,9 @@ struct gps_device_t {
 	    int part, await;		/* for tracking GSV parts */
 	    struct tm date;
 	    double subseconds;
+# ifdef NTPSHM_ENABLE
+         double last_fixtime;		/* so updates happen once */
+# endif /* NTPSHM_ENABLE */
 	} nmea;
 #endif /* NMEA_ENABLE */
 #ifdef BINARY_ENABLE
@@ -171,13 +176,11 @@ struct gps_device_t {
 #define SIRF_GE_232     0x04            /* SiRF at firmware rev >= 232 */
 #define UBLOX   	0x08		/* uBlox firmware with packet 0x62 */
 	    unsigned long satcounter;
-#ifdef NTPSHM_ENABLE
 	    unsigned int time_seen;
 #define TIME_SEEN_GPS_1	0x01	/* Seen GPS time variant 1? */
 #define TIME_SEEN_GPS_2	0x02	/* Seen GPS time variant 2? */
 #define TIME_SEEN_UTC_1	0x04	/* Seen UTC time variant 1? */
 #define TIME_SEEN_UTC_2	0x08	/* Seen UTC time variant 2? */
-#endif /* NTPSHM_ENABLE */
 	} sirf;
 #endif /* SIRFII_ENABLE */
 #ifdef TSIP_ENABLE
@@ -187,6 +190,7 @@ struct gps_device_t {
 	    time_t last_41;		/* Timestamps for packet requests */
 	    time_t last_5c;
 	    time_t last_6d;
+	    time_t last_46;
 	    unsigned int parity, stopbits; /* saved RS232 link parameters */
 	} tsip;
 #endif /* TSIP_ENABLE */
@@ -247,12 +251,18 @@ extern ssize_t packet_parse(struct gps_device_t *, size_t);
 extern ssize_t packet_get(struct gps_device_t *);
 extern int packet_sniff(struct gps_device_t *);
 
+extern int dgnss_open(struct gps_context_t *, char *);
+extern void dgnss_poll(struct gps_context_t *);
+extern void dgnss_report(struct gps_device_t *);
+extern void dgnss_autoconnect(struct gps_context_t *, double, double);
+extern void rtcm_relay(struct gps_device_t *);
+
 extern int dgpsip_open(struct gps_context_t *, const char *);
-extern void dgpsip_poll(struct gps_context_t *);
-extern void dgpsip_relay(struct gps_device_t *);
 extern void dgpsip_report(struct gps_device_t *);
 extern void dgpsip_autoconnect(struct gps_context_t *, 
 			       double, double, const char *);
+extern int ntrip_open(struct gps_context_t *, char *);
+extern void ntrip_report(struct gps_device_t *);
 
 extern int gpsd_open(struct gps_device_t *);
 extern bool gpsd_write(struct gps_device_t *, void const *, size_t);
@@ -265,6 +275,7 @@ extern void gpsd_close(struct gps_device_t *);
 extern void gpsd_zero_satellites(/*@out@*/struct gps_data_t *sp)/*@modifies sp@*/;
 extern void gpsd_interpret_subframe(struct gps_device_t *, unsigned int[]);
 extern /*@ observer @*/ char *gpsd_hexdump(const void *, size_t);
+extern void gpsd_position_fix_dump(struct gps_device_t *, /*@out@*/char[], size_t);
 extern int netlib_connectsock(const char *, const char *, const char *);
 
 extern int ntpshm_init(struct gps_context_t *, bool);
@@ -329,12 +340,16 @@ extern float roundf(float x);
 #define roundf(x) ((float)rintf(x))
 #endif /* !HAVE_ROUND */
 
-/* OpenBSD and FreeBSD don't seem to have NAN, NetBSD does, others? */
+/* OpenBSD and FreeBSD and Cygwin don't seem to have NAN, NetBSD does, others? */
 /* XXX test for this in configure? */
-#if defined(__OpenBSD__) || defined(__FreeBSD__)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__CYGWIN__)
 #ifndef NAN
 #define NAN (0.0/0.0)
 #endif /* !NAN */
 #endif /* list of Operating Systems */
 
+/* Cygwin, in addition to NAN, doesn't have cfmakeraw */
+#if defined(__CYGWIN__)
+void cfmakeraw(struct termios *);
+#endif /* defined(__CYGWIN__) */
 #endif /* _gpsd_h_ */
