@@ -20,13 +20,8 @@
  * just needs to be linked to netlib like this:
  *	gcc gpspipe.c ../netlib.c -o gpspipe
  *
- * TODO
- *      add "-p [device]" 
- *          This would force gpspipe to create a bidirectional pipe device
- *          for output.  Then programs that expect to connect to a raw GPS
- *          device could conenct to that.
  *
-  $Id: gpspipe.c 3152 2005-08-19 07:51:18Z garyemiller $
+  $Id: gpspipe.c 3185 2005-09-01 02:57:46Z esr $
  */
 
 #include <errno.h>
@@ -36,16 +31,74 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdbool.h>
+#include <fcntl.h>
+#include <termios.h>
 #include "gpsd.h"
 
 
-static void usage(const char *prog) {
+/* NMEA-0183 standard baud rate */
+#define BAUDRATE B4800
+
+/* POSIX compliant source */
+#define _POSIX_SOURCE 1
+
+/* Serial port vars */
+static struct termios oldtio,newtio;
+static int fd;
+#define SERBUF 255
+static char serbuf[SERBUF];
+
+/* open the serial port and set it up */
+static int open_serial(char* device) {
+
+	/*  Open modem device for reading and writing and not as controlling
+	 *  tty. */
+	if((fd = open(device, O_RDWR|O_NOCTTY)) < 0) {
+		fprintf(stderr,"Error opening serial port\n");
+		exit(1);
+	}
+
+	/* Save current serial port settings for later */
+	if ( tcgetattr(fd, &oldtio) != 0 ) {
+		fprintf(stderr,"Error reading serial port settings\n");
+		exit(1);
+	}
+
+	/* Clear struct for new port settings. */
+	/*@i@*/bzero(&newtio, sizeof(newtio));
+
+	/* make it raw */
+	(void)cfmakeraw(&newtio);
+	/* set speed */
+	/*@i@*/(void)cfsetospeed(&newtio, BAUDRATE);
+	 
+	/* Clear the modem line and activate the settings for the port. */
+	(void)tcflush(fd,TCIFLUSH);
+	if ( tcsetattr(fd,TCSANOW,&newtio) != 0 ) {
+		(void)fprintf(stderr,"Error setting serial port settings\n");
+		exit(1);
+	}
+
+	return(fd);
+}
+
+
+/* Send a string to the serial port. */
+static int send_string(char *buf, size_t len) {
+
+	/* Send the string and  Return the result. */
+	return ((int)write(fd, buf, len ) );
+}
+
+
+static void usage(void) {
 	fprintf(stderr, "Usage: gpspipe [OPTIONS] [server[:port]]\n\n"
-	        "SVN ID: $Id: gpspipe.c 3152 2005-08-19 07:51:18Z garyemiller $ \n"
+	        "SVN ID: $Id: gpspipe.c 3185 2005-09-01 02:57:46Z esr $ \n"
 		"-h show this help\n"
 		"-r Dump raw NMEA\n"
 	        "-w Dump gpsd native data\n"
 	        "-t time stamp the data\n"
+		"-s [serial dev] emulate a 4800bps NMEA GPS on serial port (use with '-r')\n"
 	        "-n [count] exit after count packets\n"
 	        "-V print version and exit\n\n"
 	        "You must specify one, or both, of -r/-w\n"
@@ -68,8 +121,9 @@ int main( int argc, char **argv) {
         char *port = DEFAULT_GPSD_PORT, *server = "127.0.0.1";
 	//extern char *optarg;
 
+	char *serialport = NULL;
 
-	while ((option = getopt(argc, argv, "?hrwtVn:")) != -1) {
+	while ((option = getopt(argc, argv, "?hrwtVn:s:")) != -1) {
 		switch (option) {
 		case 'n':
 			count = strtol(optarg, 0, 0);
@@ -84,15 +138,24 @@ int main( int argc, char **argv) {
 			dump_gpsd = true;
 			break;
 		case 'V':
-	                (void)fprintf(stderr, "%s: SVN ID: $Id: gpspipe.c 3152 2005-08-19 07:51:18Z garyemiller $ \n", argv[0]);
+	                (void)fprintf(stderr, "%s: SVN ID: $Id: gpspipe.c 3185 2005-09-01 02:57:46Z esr $ \n", argv[0]);
 			exit(0);
+		case 's':
+		        serialport = optarg;
+		        break;
 		case '?':
 		case 'h':
 		default:
-			usage( argv[0] );
+			usage( );
 			exit(1);
 		}
 	}
+
+	if( serialport != NULL && !dump_nmea ) {
+	  fprintf(stderr,"Use of '-s' requires '-r'\n");
+	  exit(1);
+	}
+
 	if ( dump_nmea ) {
 		if ( dump_gpsd ) {
 			cstr = "rw\n";
@@ -102,7 +165,7 @@ int main( int argc, char **argv) {
 	} else if ( dump_gpsd ) {
 		cstr = "w\n";
 	} else {
-		usage( argv[0] );
+		usage( );
 		exit(1);
 	}
 	/* Grok the server, port, and device. */
@@ -132,12 +195,19 @@ int main( int argc, char **argv) {
 	}
 	/*@ +branchstate @*/
 
+	/* Open the serial port and set it up. */
+	if(serialport) {
+	    (int)open_serial(serialport);
+	}
+
+	/*@ -nullpass @*/
 	s = netlib_connectsock( server, port, "tcp");
 	if ( s < 0 ) {
 		fprintf( stderr, "%s: could not connect to gpsd %s:%s, %s(%d)\n"
 			, argv[0] , server, port, strerror(errno), errno);
 		exit (1);
 	}
+	/*@ +nullpass @*/
 
 	wrote = write( s, cstr, strlen(cstr) );
 	if ( (ssize_t)strlen(cstr) != wrote ) {
@@ -148,12 +218,16 @@ int main( int argc, char **argv) {
 
 	for(;;) {
 		int i = 0;
+		int j = 0;
 		int readbytes = 0;
 
 		readbytes = (int)read( s, buf, sizeof(buf));
 		if ( readbytes > 0 ) {
 		    for ( i = 0 ; i < readbytes ; i++ ) {
 			char c = buf[i];
+			if( j < (SERBUF - 1) ) {
+				serbuf[j++] = buf[i];
+			}
 			if ( new_line && timestamp ) {
 				time_t now = time(NULL);
 
@@ -173,10 +247,22 @@ int main( int argc, char **argv) {
 					, strerror(errno), errno);
 				exit (1);
 			}
+
 		
 			if ( c == '\n' ) {
+
+			    if( serialport != NULL) {
+			      if ( -1 == send_string( serbuf, (size_t)j )) {
+			        fprintf( stderr, "%s: Serial port write Error, %s(%d)\n"
+				       , argv[0]
+				       , strerror(errno), errno);
+			        exit (1);
+			      }
+			      j = 0;
+			    }
+
 			    new_line = true;
-			    /* flush after eveery good line */
+			    /* flush after every good line */
 			    if (  fflush( stdout ) ) {
 				fprintf( stderr, "%s: fflush Error, %s(%d)\n"
 					, argv[0]
@@ -199,5 +285,16 @@ int main( int argc, char **argv) {
 			exit(1);
 		}
 	}
-	//exit(0);
+
+#ifdef __UNUSED__
+	if( serialport != NULL ) {
+		/* Restore the old serial port settings. */
+		if ( tcsetattr(fd, TCSANOW, &oldtio) != 0 ) {
+			fprintf(stderr, "Error restoring serial port settings\n");
+			exit(1);
+		}
+	}
+
+	exit(0);
+#endif /* __UNUSED__ */  
 }
