@@ -183,10 +183,11 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 
     for (ns = buf; ns; ns = strstr(ns+1, "GPSD")) {
 	if (/*@i1@*/strncmp(ns, "GPSD", 4) == 0) {
-	    for (sp = ns + 5; ; sp = tp) {
+	    /* the following should execute each time we have a good next sp */
+	    for (sp = ns + 5; *sp != '\0'; sp = tp+1) {
 		tp = sp + strcspn(sp, ",\r\n");
-		if (*tp == '\0') break;
-		*tp = '\0';
+		if (*tp == '\0') tp--;
+		else *tp = '\0';
 
 		switch (*sp) {
 		case 'A':
@@ -245,13 +246,22 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    }
 		    /*@ +mustfreeonly */
 		    break;
+#ifdef HEADING_FIX
+		case 'H':
+		    if (sp[2] == '?') {
+		        gpsdata->fix.heading = NAN;
+		    } else {
+		        (void)sscanf(sp, "H=%lf", &gpsdata->fix.heading);
+		    }
+		    break;
+#endif /* HEADING_FIX */
 		case 'I':
 		    /*@ -mustfreeonly */
-		    if (sp[2] == '?') 
+		    if (gpsdata->gps_id)
+			free(gpsdata->gps_id);
+		    if (sp[2] == '?')
 			gpsdata->gps_id = NULL;
 		    else {
-			if (gpsdata->gps_id)
-			    free(gpsdata->gps_id);
 			gpsdata->gps_id = strdup(sp+2);
 			gpsdata->set |= DEVICEID_SET;
 		    }
@@ -272,11 +282,11 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			gpsdata->devicelist = (char **)calloc(
 			    (size_t)gpsdata->ndevices,
 			    sizeof(char **));
-			/*@ -nullstate @*/
-			gpsdata->devicelist[i=0] = strtok_r(sp+2, " \r\n", &ns);
+			/*@ -nullstate -mustfreefresh @*/
+			gpsdata->devicelist[i=0] = strdup(strtok_r(sp+2, " \r\n", &ns));
 			while ((sp = strtok_r(NULL, " \r\n",  &ns)))
 			    gpsdata->devicelist[++i] = strdup(sp);
-			/*@ +nullstate @*/
+			/*@ +nullstate +mustfreefresh @*/
 			/*@ +nullderef @*/
 			gpsdata->set |= DEVICELIST_SET;
 		    }
@@ -304,14 +314,14 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			struct gps_fix_t nf;
 			char tag[MAXTAGLEN+1], alt[20];
 			char eph[20], epv[20], track[20],speed[20], climb[20];
-			char epd[20], eps[20], epc[20];
+			char epd[20], eps[20], epc[20], mode[2];
 			int st = sscanf(sp+2, 
-			       "%8s %lf %lf %lf %lf %s %s %s %s %s %s %s %s %s",
+			       "%8s %lf %lf %lf %lf %s %s %s %s %s %s %s %s %s %s",
 				tag, &nf.time, &nf.ept, 
 				&nf.latitude, &nf.longitude,
 			        alt, eph, epv, track, speed, climb,
-			        epd, eps, epc);
-			if (st == 14) {
+			        epd, eps, epc, mode);
+			if (st == 15) {
 #define DEFAULT(val) (val[0] == '?') ? NAN : atof(val)
 			    /*@ +floatdouble @*/
 			    nf.altitude = DEFAULT(alt);
@@ -325,8 +335,11 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			    nf.epc = DEFAULT(epc);
 			    /*@ -floatdouble @*/
 #undef DEFAULT
-			    nf.mode = (alt[0] == '?') ? MODE_2D : MODE_3D;
-			    if (nf.mode == MODE_3D)
+			    if (st >= 15)
+				nf.mode = (mode[0] == '?') ? MODE_NOT_SEEN : atoi(mode);
+			    else
+				nf.mode = (alt[0] == '?') ? MODE_2D : MODE_3D;
+			    if (alt[0] != '?')
 				gpsdata->set |= ALTITUDE_SET | CLIMB_SET;
 			    if (isnan(nf.eph)==0)
 				gpsdata->set |= HERR_SET;
@@ -338,7 +351,12 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 				gpsdata->set |= SPEEDERR_SET;
 			    if (isnan(nf.epc)==0)
 				gpsdata->set |= CLIMBERR_SET;
+#ifndef HEADING_FIX
 			    nf.pitch = nf.roll = nf.dip = NAN;
+#else /* HEADING_FIX */
+			    /* there is no support for a TrueNorth source in the O command at this stage, so libgps-based clients that wish to use the heading provided by such a source must send the sequence "OH\n".  N.B. the order is important - "HO\n" will result in heading=NAN.  More work is needed to provide any support for pitch, roll and dip. */
+			    nf.pitch = nf.roll = nf.dip = nf.heading = NAN;
+#endif /* HEADING_FIX */
 			    gpsdata->fix = nf;
 			    (void)strcpy(gpsdata->tag, tag);
 			    gpsdata->set |= TIME_SET|TIMERR_SET|LATLON_SET|MODE_SET;
@@ -403,6 +421,8 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		        gpsdata->fix.speed = NAN;
 		    } else {
 		        (void)sscanf(sp, "V=%lf", &gpsdata->fix.speed);
+			/* V reply is in kt, fix.speed is in metres/sec */
+			gpsdata->fix.speed = gpsdata->fix.speed / MPS_TO_KNOTS;
 		        gpsdata->set |= SPEED_SET;
 		    }
 		    break;
@@ -434,12 +454,14 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			for (j = 0; j < gpsdata->satellites; j++) {
 			    PRN[j]=elevation[j]=azimuth[j]=ss[j]=used[j]=0;
 			}
-			for (j = 0; j < gpsdata->satellites; j++) {
+			for (j = 0, gpsdata->satellites_used = 0; j < gpsdata->satellites; j++) {
 			    sp = strchr(sp, ':') + 1;
 			    (void)sscanf(sp, "%d %d %d %d %d", &i1, &i2, &i3, &i4, &i5);
 			    PRN[j] = i1;
 			    elevation[j] = i2; azimuth[j] = i3;
 			    ss[j] = i4; used[j] = i5;
+			    if (i5 == 1)
+                                gpsdata->satellites_used++;
 			}
 			/*@ -compdef @*/
 			memcpy(gpsdata->PRN, PRN, sizeof(PRN));
@@ -455,8 +477,8 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    gpsdata->profiling = (sp[2] == '1');
 		    break;
 		case '$':
-		    /*@ +matchanyintegral @*/
-		    (void)sscanf(sp, "$=%s %ld %lf %lf %lf %lf %lf %lf", 
+		    /*@ +matchanyintegral -formatcode @*/
+		    (void)sscanf(sp, "$=%s %zd %lf %lf %lf %lf %lf %lf", 
 			   gpsdata->tag,
 			   &gpsdata->sentence_length,
 			   &gpsdata->fix.time, 
@@ -465,7 +487,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			   &gpsdata->d_decode_time, 
 			   &gpsdata->poll_time, 
 			   &gpsdata->emit_time);
-		    /*@ -matchanyintegral @*/
+		    /*@ -matchanyintegral +formatcode @*/
 		    break;
 		}
 	    }
@@ -528,8 +550,10 @@ static void *poll_gpsd(void *args)
 
     /* set thread parameters */
     /*@ -compdef @*/
+    /*@ -unrecog (splint has no pthread declarations as yet) @*/
     (void)pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,&oldstate);
     (void)pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,&oldtype); /* we want to be canceled also when blocked on gps_poll() */
+    /*@ +unrecog @*/
     /*@ +compdef @*/
     gpsdata = (struct gps_data_t *) args;
     do {
@@ -553,7 +577,9 @@ int gps_set_callback(struct gps_data_t *gpsdata,
     gpsdata->thread_hook = callback;
 
     /* start the thread which will read data from gpsd */
+    /*@ -unrecog (splint has no pthread declarations as yet */
     return pthread_create(handler,NULL,poll_gpsd,(void*)gpsdata);
+    /*@ +unrecog @*/
 }
 
 int gps_del_callback(struct gps_data_t *gpsdata, pthread_t *handler)
@@ -572,7 +598,7 @@ int gps_del_callback(struct gps_data_t *gpsdata, pthread_t *handler)
  * A simple command-line exerciser for the library.
  * Not really useful for anything but debugging.
  */
-void data_dump(struct gps_data_t *collect, time_t now)
+static void data_dump(struct gps_data_t *collect, time_t now)
 {
     char *status_values[] = {"NO_FIX", "FIX", "DGPS_FIX"};
     char *mode_values[] = {"", "NO_FIX", "MODE_2D", "MODE_3D"};
@@ -620,18 +646,18 @@ void data_dump(struct gps_data_t *collect, time_t now)
 	
 }
 
-static void dumpline(struct gps_data_t *ud UNUSED, char *buf)
+static void dumpline(struct gps_data_t *ud UNUSED, char *buf,
+		     size_t ulen UNUSED, int level UNUSED)
 {
     puts(buf);
 }
 
 #include <getopt.h>
 
-main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     struct gps_data_t *collect;
-    char buf[BUFSIZ], *device = NULL;
-    int option;
+    char buf[BUFSIZ];
 
     collect = gps_open(NULL, 0);
     gps_set_raw_hook(collect, dumpline);
@@ -660,6 +686,7 @@ main(int argc, char *argv[])
     }
 
     (void)gps_close(collect);
+    return 0;
 }
 
 #endif /* TESTMAIN */
