@@ -14,7 +14,9 @@
 #include "gpsd.h"
 
 #if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
-#include <pthread.h>
+#ifndef S_SPLINT_S
+#include <pthread.h>	/* pacifies OpenBSD's compiler */
+#endif
 #endif
 
 
@@ -211,8 +213,9 @@ static double degtodm(double a)
     return t;
 }
 
-static void gpsd_binary_fix_dump(struct gps_device_t *session, 
-				 char bufp[],size_t len)
+/*@ -mustdefine @*/
+void gpsd_position_fix_dump(struct gps_device_t *session,
+			    /*@out@*/char bufp[], size_t len)
 {
     struct tm tm;
     time_t intfixtime;
@@ -255,10 +258,20 @@ static void gpsd_binary_fix_dump(struct gps_device_t *session,
 	    (void)strcat(bufp, (session->mag_var > 0) ? "E": "W");
 	}
 	nmea_add_checksum(bufp);
-	bufp += strlen(bufp);
     }
+}
+/*@ +mustdefine @*/
+
+static void gpsd_transit_fix_dump(struct gps_device_t *session,
+				  char bufp[], size_t len)
+{
+    struct tm tm;
+    time_t intfixtime;
+
+    intfixtime = (time_t)session->gpsdata.fix.time;
+    (void)gmtime_r(&intfixtime, &tm);
     /*@ -usedef @*/
-    (void)snprintf(bufp, len-strlen(bufp),
+    (void)snprintf(bufp, len,
 	    "$GPRMC,%02d%02d%02d,%c,%09.4f,%c,%010.4f,%c,%.4f,%.3f,%02d%02d%02d,,",
 	    tm.tm_hour, 
 	    tm.tm_min, 
@@ -268,7 +281,7 @@ static void gpsd_binary_fix_dump(struct gps_device_t *session,
 	    ((session->gpsdata.fix.latitude > 0) ? 'N' : 'S'),
 	    degtodm(fabs(session->gpsdata.fix.longitude)),
 	    ((session->gpsdata.fix.longitude > 0) ? 'E' : 'W'),
-	    session->gpsdata.fix.speed,
+	    session->gpsdata.fix.speed * MPS_TO_KNOTS,
 	    session->gpsdata.fix.track,
 	    tm.tm_mday,
 	    tm.tm_mon + 1,
@@ -277,6 +290,13 @@ static void gpsd_binary_fix_dump(struct gps_device_t *session,
     nmea_add_checksum(bufp);
 }
 
+static void gpsd_binary_fix_dump(struct gps_device_t *session,
+				 char bufp[], size_t len)
+{
+    gpsd_position_fix_dump(session, bufp, len);
+    gpsd_transit_fix_dump(session, bufp + strlen(bufp), len - strlen(bufp));
+}
+
 static void gpsd_binary_satellite_dump(struct gps_device_t *session, 
 				char bufp[], size_t len)
 {
@@ -634,7 +654,7 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	    }
 	}
 
-	dgpsip_report(session);
+	dgnss_report(session);
 
 	return session->gpsdata.set;
     }
