@@ -21,11 +21,12 @@ int netlib_connectsock(const char *host, const char *service, const char *protoc
     struct sockaddr_in sin;
     int s, type, one = 1;
 
-    memset((char *) &sin, '\0', sizeof(sin));
+    memset((char *) &sin, 0, sizeof(sin));
+    /*@ -type -mustfreefresh @*/
     sin.sin_family = AF_INET;
     if ((pse = getservbyname(service, protocol)))
-	sin.sin_port = htons(ntohs((u_short) pse->s_port));
-    else if ((sin.sin_port = htons((u_short) atoi(service))) == 0)
+	sin.sin_port = htons(ntohs((unsigned short) pse->s_port));
+    else if ((sin.sin_port = htons((unsigned short) atoi(service))) == 0)
 	return NL_NOSERVICE;
     if ((phe = gethostbyname(host)))
 	memcpy((char *) &sin.sin_addr, phe->h_addr, phe->h_length);
@@ -41,13 +42,14 @@ int netlib_connectsock(const char *host, const char *service, const char *protoc
     if ((s = socket(PF_INET, type, ppe->p_proto)) < 0)
 	return NL_NOSOCK;
     if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one))==-1) {
-        close(s);
+        (void)close(s);
 	return NL_NOSOCKOPT;
     }
     if (connect(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
-        close(s);
+        (void)close(s);
 	return NL_NOCONNECT;
     }
     return s;
+    /*@ +type +mustfreefresh @*/
 }
 
