@@ -258,6 +258,9 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[], struct gps_device_t
 	else {
 	    mask |= TIME_SET;
 	    session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
+	    if (session->gpsdata.sentence_time != session->gpsdata.newdata.time)
+		mask |= CYCLE_START_SET;
+	    session->gpsdata.sentence_time = session->gpsdata.newdata.time;
 	}
 	do_lat_lon(&field[2], &session->gpsdata);
 	mask |= LATLON_SET;
@@ -331,7 +334,15 @@ static gps_mask_t processGPGSA(int count, char *field[], struct gps_device_t *se
 	return ONLINE_SET;
 
     session->gpsdata.newdata.mode = atoi(field[2]);
-    mask = MODE_SET;
+    /*
+     * The first arm of this conditional ignores dead-reckoning
+     * fixes from an Antaris chipset. which returns E in field 2
+     * for a dead-reckoning estimate.  Fix by Andreas Stricker.
+     */
+    if (session->gpsdata.newdata.mode == 0 && field[2][0] == 'E')
+        mask = 0;
+    else
+        mask = MODE_SET;
     gpsd_report(3, "GPGSA sets mode %d\n", session->gpsdata.newdata.mode);
     session->gpsdata.pdop = atof(field[session->device_type->channels+3]);
     session->gpsdata.hdop = atof(field[session->device_type->channels+4]);
@@ -500,9 +511,17 @@ static gps_mask_t processTNTHTM(int c UNUSED, char *field[], struct gps_device_t
 
     //gpsd_zero_satellites(&session->gpsdata);
 
+#ifdef HEADING_FIX
+	// these fields are not handled by gps_merge_fix(), so must be written directly to the current fix structure
+#endif /* HEADING_FIX */
     session->gpsdata.fix.time = timestamp();
+#ifndef HEADING_FIX
     session->gpsdata.newdata.track = atof(field[1]);
+#else /* HEADING_FIX */
+    session->gpsdata.fix.heading = atof(field[1]);
+#endif /* HEADING_FIX */
     session->gpsdata.headingStatus = *field[2];
+#ifndef HEADING_FIX
     mask |= (TRACK_SET | MODE_SET);
     session->gpsdata.status = STATUS_FIX;	/* could be DGPS_FIX, we can't tell */
     //session->gpsdata.newdata.mode = MODE_2D;
@@ -510,15 +529,35 @@ static gps_mask_t processTNTHTM(int c UNUSED, char *field[], struct gps_device_t
     //session->gpsdata.newdata.mode = MODE_NO_FIX;
 
     session->gpsdata.newdata.pitch = atof(field[3]);
+#else /* HEADING_FIX */
+    // this device does not provide track, mode, or status - we have to rely on other devices for that!
+//    mask |= (TRACK_SET | MODE_SET);
+//    session->gpsdata.status = STATUS_FIX;	/* could be DGPS_FIX, we can't tell */
+//    session->gpsdata.newdata.mode = MODE_2D;
+    
+    session->gpsdata.fix.pitch = atof(field[3]);
+#endif /* HEADING_FIX */
     session->gpsdata.pitchStatus = *field[4];
 
+#ifndef HEADING_FIX
     session->gpsdata.newdata.roll = atof(field[5]);
+#else /* HEADING_FIX */
+    session->gpsdata.fix.roll = atof(field[5]);
+#endif /* HEADING_FIX */
     session->gpsdata.rollStatus = *field[6];
 
+#ifndef HEADING_FIX
     session->gpsdata.newdata.dip = atof(field[7]);
+#else /* HEADING_FIX */
+    session->gpsdata.fix.dip = atof(field[7]);
+#endif /* HEADING_FIX */
     session->gpsdata.horzField = atof(field[8]);
 
+#ifndef HEADING_FIX
     //gpsd_report(5, "Heading %lf  %c.\n", session->gpsdata.fix.track, session->gpsdata.fix.headingStatus);
+#else /* HEADING_FIX */
+    //gpsd_report(5, "Heading %lf  %c.\n", session->gpsdata.fix.heading, session->gpsdata.fix.headingStatus);
+#endif /* HEADING_FIX */
     return mask;
 }
 #endif /* TNT_ENABLE */
