@@ -7,7 +7,7 @@ DESCRIPTION:
 
 Initial conditions of the problem:
 
-1. We have a file descriptor open for (possibly non-blockin) read. The device 
+1. We have a file descriptor open for (possibly non-blocking) read. The device 
    on the other end is sending packets at us.  
 
 2. It may require more than one read to gather a packet.  Reads may span packet
@@ -814,7 +814,9 @@ void packet_reset(struct gps_device_t *session)
     session->packet_state = GROUND_STATE;
     session->inbuflen = 0;
     session->inbufptr = session->inbuffer;
+#ifdef BINARY_ENABLE
     isgps_init(session);
+#endif /* BINARY_ENABLE */
 }
 
 
