@@ -9,7 +9,7 @@ RTCM RECOMMENDED STANDARDS FOR DIFFERENTIAL NAVSTAR GPS SERVICE,
 RTCM PAPER 194-93/SC 104-STD
 
 Ordering instructions are accessible from <http://www.rtcm.org/>
-under "Publications".
+under "Publications".  This describes version 2.1 of the RTCM specification.
 
 Also applicable is ITU-R M.823: "Technical characteristics of
 differential transmissions for global navigation satellite systems
@@ -56,6 +56,8 @@ Starlink's website.
 
 #include "gpsd.h"
 
+#ifdef RTCM104_ENABLE
+
 /*
  * Structures for interpreting words in an RTCM-104 message (after
  * parity checking and removing inversion).
@@ -1179,3 +1181,5 @@ void rtcm_output_mag(isgps30bits_t * ip)
     }
 }
 #endif /* UNUSED */
+
+#endif /* RTCM104_ENABLE */
