@@ -17,6 +17,19 @@
 #  endif /* CNEW_RTSCTS */
 #endif /* !CRTSCTS */
 
+#if defined(__CYGWIN__)
+/* Workaround for Cygwin, which is missing cfmakeraw */
+/* Pasted from man page; added in serial.c arbitrarily */
+void cfmakeraw(struct termios *termios_p)
+{
+    termios_p->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
+    termios_p->c_oflag &= ~OPOST;
+    termios_p->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
+    termios_p->c_cflag &= ~(CSIZE|PARENB);
+    termios_p->c_cflag |= CS8;
+}
+#endif /* defined(__CYGWIN__) */
+
 speed_t gpsd_get_speed(struct termios* ttyctl)
 {
     speed_t code = cfgetospeed(ttyctl);
@@ -97,13 +110,18 @@ void gpsd_set_speed(struct gps_device_t *session,
 int gpsd_open(struct gps_device_t *session)
 {
     gpsd_report(1, "opening GPS data source at '%s'\n", session->gpsdata.gps_device);
-    if ((session->gpsdata.gps_fd = open(session->gpsdata.gps_device, O_RDWR|O_NOCTTY)) < 0) {
+    if ((session->gpsdata.gps_fd = open(session->gpsdata.gps_device, O_RDWR|O_NONBLOCK|O_NOCTTY)) < 0) {
 	gpsd_report(1, "device open failed: %s\n", strerror(errno));
 	return -1;
     }
 
+#ifdef FIXED_PORT_SPEED
+    session->saved_baud = FIXED_PORT_SPEED;
+#endif
+
     if (session->saved_baud != -1) {
-        /*@i@*/(void)cfsetspeed(&session->ttyset, (speed_t)session->saved_baud);
+        /*@i@*/(void)cfsetispeed(&session->ttyset, (speed_t)session->saved_baud);
+        /*@i@*/(void)cfsetospeed(&session->ttyset, (speed_t)session->saved_baud);
 	(void)tcsetattr(session->gpsdata.gps_fd, TCSANOW, &session->ttyset);
 	(void)tcflush(session->gpsdata.gps_fd, TCIOFLUSH);
     }
@@ -179,8 +197,13 @@ bool gpsd_write(struct gps_device_t *session, void const *buf, size_t len)
 bool gpsd_next_hunt_setting(struct gps_device_t *session)
 /* advance to the next hunt setting  */
 {
+#ifdef FIXED_PORT_SPEED
+    /* just the one fixed port speed... */
+    static unsigned int rates[] = {FIXED_PORT_SPEED};
+#else /* FIXED_PORT_SPEED not defined */
     /* every rate we're likely to see on a GPS */
     static unsigned int rates[] = {0, 4800, 9600, 19200, 38400, 57600};
+#endif /* FIXED_PORT_SPEED defined */
 
     if (session->retry_counter++ >= SNIFF_RETRIES) {
 	session->retry_counter = 0;
@@ -195,6 +218,7 @@ bool gpsd_next_hunt_setting(struct gps_device_t *session)
     }
 
     return true;	/* keep hunting */
+
 }
 
 void gpsd_close(struct gps_device_t *session)
@@ -215,3 +239,4 @@ void gpsd_close(struct gps_device_t *session)
 	session->gpsdata.gps_fd = -1;
     }
 }
+
