@@ -73,7 +73,7 @@ static bool sirf_speed(int ttyfd, speed_t speed)
 /* change speed in binary mode */
 {
     /*@ +charint @*/
-   unsigned char msg[] = {0xa0, 0xa2, 0x00, 0x09,
+   static unsigned char msg[] = {0xa0, 0xa2, 0x00, 0x09,
                      0x86, 
                      0x0, 0x0, 0x12, 0xc0,	/* 4800 bps */
 		     0x08,			/* 8 data bits */
@@ -92,7 +92,7 @@ static bool sirf_to_nmea(int ttyfd, speed_t speed)
 /* switch from binary to NMEA at specified baud */
 {
     /*@ +charint @*/
-   unsigned char msg[] = {0xa0, 0xa2, 0x00, 0x18,
+   static unsigned char msg[] = {0xa0, 0xa2, 0x00, 0x18,
                      0x81, 0x02,
                      0x01, 0x01, /* GGA */
                      0x00, 0x00, /* suppress GLL */
@@ -128,7 +128,7 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
     char buf2[MAX_PACKET_LENGTH*3+2];
     double fv;
     /*@ +charint @*/
-    unsigned char enablesubframe[] = {0xa0, 0xa2, 0x00, 0x19,
+    static unsigned char enablesubframe[] = {0xa0, 0xa2, 0x00, 0x19,
 				 0x80, 0x00, 0x00, 0x00,
 				 0x00, 0x00, 0x00, 0x00,
 				 0x00, 0x00, 0x00, 0x00,
@@ -137,7 +137,7 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 				 0x00, 0x00, 0x00, 0x0C,
 				 0x10,
 				 0x00, 0x00, 0xb0, 0xb3};
-    unsigned char disablesubframe[] = {0xa0, 0xa2, 0x00, 0x19,
+    static unsigned char disablesubframe[] = {0xa0, 0xa2, 0x00, 0x19,
 				  0x80, 0x00, 0x00, 0x00,
 				  0x00, 0x00, 0x00, 0x00,
 				  0x00, 0x00, 0x00, 0x00,
@@ -660,18 +660,18 @@ static void sirfbin_initializer(struct gps_device_t *session)
     /* do this every time*/
     {
 	/*@ +charint @*/
-	unsigned char dgpscontrol[] = {0xa0, 0xa2, 0x00, 0x07,
+	static unsigned char dgpscontrol[] = {0xa0, 0xa2, 0x00, 0x07,
 				 0x85, 0x01, 0x00, 0x00,
 				 0x00, 0x00, 0x00,
 				 0x00, 0x00, 0xb0, 0xb3};
-	unsigned char sbasparams[] = {0xa0, 0xa2, 0x00, 0x06,
+	static unsigned char sbasparams[] = {0xa0, 0xa2, 0x00, 0x06,
 				 0xaa, 0x00, 0x01, 0x00,
 				 0x00, 0x00,
 				 0x00, 0x00, 0xb0, 0xb3};
-	unsigned char versionprobe[] = {0xa0, 0xa2, 0x00, 0x02,
+	static unsigned char versionprobe[] = {0xa0, 0xa2, 0x00, 0x02,
 				 0x84, 0x00,
 				 0x00, 0x00, 0xb0, 0xb3};
-	unsigned char modecontrol[] = {0xa0, 0xa2, 0x00, 0x0e,
+	static unsigned char modecontrol[] = {0xa0, 0xa2, 0x00, 0x0e,
 				  0x88, 
 				  0x00, 0x00,	/* pad bytes */
 				  0x00,		/* degraded mode off */
@@ -684,11 +684,11 @@ static void sirfbin_initializer(struct gps_device_t *session)
 				  0x00,		/* disable dead reckoning */
 				  0x01,		/* enable track smoothing */
 				 0x00, 0x00, 0xb0, 0xb3};
-	unsigned char requestecef[] = {0xa0, 0xa2, 0x00, 0x08,
+	static unsigned char requestecef[] = {0xa0, 0xa2, 0x00, 0x08,
 				       0xa6, 0x00, 0x02, 0x01,
 				       0x00, 0x00, 0x00, 0x00,
 				       0x00, 0x00, 0xb0, 0xb3};
-	unsigned char requesttracker[] = {0xa0, 0xa2, 0x00, 0x08, 
+	static unsigned char requesttracker[] = {0xa0, 0xa2, 0x00, 0x08, 
 					  0xa6, 0x00, 0x04, 0x03,
 					  0x00, 0x00, 0x00, 0x00,
 					  0x00, 0x00, 0xb0, 0xb3};
