@@ -29,6 +29,10 @@
 #include <math.h>
 #include <ctype.h>
 #include <unistd.h>
+/* Cygwin has only _timezone and not timezone unless the following is set */
+#if defined(__CYGWIN__)
+#define timezonevar
+#endif /* defined(__CYGWIN__) */
 #include <time.h>
 #include <termios.h>
 #include <fcntl.h>	/* for O_RDWR */
@@ -921,12 +925,12 @@ int main (int argc, char **argv)
     gmt_offset = (int)tzoffset();
 
     /*@ -branchstate @*/
-    while ((option = getopt(argc, argv, "F:vh")) != -1) {
+    while ((option = getopt(argc, argv, "F:Vh")) != -1) {
 	switch (option) {
 	case 'F':
 	    controlsock = optarg;
 	    break;
-	case 'v':
+	case 'V':
 	    (void)printf("sirfmon %s\n", VERSION);
 	    exit(0);
 	case 'h': case '?': default:
