@@ -4,15 +4,23 @@
  * The current (fixed) leap-second correction, and the future Unix
  * time after which to start hunting leap-second corrections from GPS
  * subframe data if the GPS doesn't supply them any more readily.
- * Currently 1 Jan 2006, as the current fixed correction is expected
- * to be good until at least then.  Deferring the check is a hack to
- * speed up fix acquisition -- subframe data is bulky enough to
- * substantially increase latency.  To update LEAP_SECONDS and
- * START_SUBFRAME, see the IERS leap-second bulletin page at:
+ *
+ * Deferring the check is a hack to speed up fix acquisition --
+ * subframe data is bulky enough to substantially increase latency.
+ * To update LEAP_SECONDS and START_SUBFRAME, see the IERS leap-second
+ * bulletin page at:
  * <http://hpiers.obspm.fr/eop-pc/products/bulletins/bulletins.html>
+ *
+ * You can use the Python expression
+ *	time.mktime(time.strptime(... , "%d %b %Y %H:%M:%S"))
+ * to generate an integer value for START_SUBFRAME. 
  */
-#define LEAP_SECONDS	13
-#define START_SUBFRAME	1136091600
+#define LEAP_SECONDS	14
+
+/* IERS says "NO positive leap second will be introduced at the end of 
+ * June 2006, so start subframe checking at the *next* 6-month boundary.
+ */
+#define START_SUBFRAME	1167627600	/* 1 Jan 2007 00:00:00 */
 
 /*
  * This is used only when an NMEA device issues a two-digit year in a GPRMC
