@@ -25,18 +25,19 @@
  * Widget and window sizes.
  */
 #define MAX_FONTSIZE	18		/* maximum fontsize we handle*/
-#define MAX_CHANNELS	12		/* count of channels to display */
-#define ROOT_SIZE	"700x570"	/* size of entire xgps display */
 /* height of satellite-data display */
-#define SATDATA_HEIGHT	MAX_FONTSIZE*(MAX_CHANNELS+1)
+#define SATDATA_HEIGHT	MAX_FONTSIZE*(MAXCHANNELS+1)
 #define LEFTSIDE_WIDTH	205		/* width of data-display side */
 #define SATDIAG_SIZE	400		/* size of satellite diagram */
 
-static Widget toplevel, form, left, right, quitbutton;
+static Widget toplevel, form, left, right;
+#ifdef EXPLICIT_QUIT
+static Widget quitbutton;
+#endif /* EXPLICIT_QUIT */
 static Widget satellite_list, satellite_diagram, status;
 static Widget rowColumn_11, rowColumn_12, rowColumn_13, rowColumn_14;
 static Widget rowColumn_15, rowColumn_16, rowColumn_17, rowColumn_18;
-static Widget rowColumn_19, rowColumn_20, rowColumn_21;
+static Widget rowColumn_19, rowColumn_20;
 static Widget text_1, text_2, text_3, text_4, text_5;
 static Widget text_6, text_7, text_8, text_9, text_10;
 static Widget label_1, label_2, label_3, label_4, label_5;
@@ -96,14 +97,20 @@ static void build_gui(Widget toplevel)
 
     /*@ -immediatetrans -usedef @*/
     /* the root application window */
-    XtSetArg(args[0], XmNgeometry, ROOT_SIZE);
-    XtSetArg(args[1], XmNresizePolicy, XmRESIZE_NONE);
-    XtSetArg(args[2], XmNallowShellResize, False);
-    XtSetArg(args[3], XmNdeleteResponse, XmDO_NOTHING);
-    XtSetArg(args[4], XmNmwmFunctions,
+    XtSetArg(args[0], XmNwidth, LEFTSIDE_WIDTH + SATDIAG_SIZE + 26);
+    XtSetArg(args[1], XmNheight, SATDATA_HEIGHT + 14*MAX_FONTSIZE + 12);
+#ifdef __UNUSED__
+    XtSetArg(args[2], XmNresizePolicy, XmRESIZE_NONE);
+    XtSetArg(args[3], XmNallowShellResize, False);
+    XtSetArg(args[4], XmNdeleteResponse, XmDO_NOTHING);
+    XtSetArg(args[5], XmNmwmFunctions,
 	     MWM_FUNC_RESIZE | MWM_FUNC_MOVE | MWM_FUNC_MINIMIZE | MWM_FUNC_MAXIMIZE);
     /*@ +immediatetrans +usedef @*/
-    XtSetValues(toplevel, args, 5);
+    XtSetValues(toplevel, args, 6);
+#else
+    /*@ +immediatetrans +usedef @*/
+    XtSetValues(toplevel, args, 2);
+#endif /* __UNUSED__ */
 
     /*@ -onlytrans @*/
     /* a form to assist with geometry negotiation */
@@ -125,7 +132,7 @@ static void build_gui(Widget toplevel)
 				     XmNeditable, False,
 				     XmNmarginHeight, 1,
 				     XmNhighlightThickness, 0,
-				     XmNshadowThickness, 1,
+				     XmNshadowThickness, 2,
 				     XmNleftAttachment, XmATTACH_FORM,
 				     XmNrightAttachment, XmATTACH_FORM,
 				     XmNtopAttachment, XmATTACH_WIDGET,
@@ -146,7 +153,7 @@ static void build_gui(Widget toplevel)
       XtVaCreateManagedWidget("satellite_diagram",
 			      xmDrawingAreaWidgetClass, right, 
 			      XmNbackground, get_pixel(toplevel, "snow"),
-			      XmNheight, SATDIAG_SIZE, XmNwidth, SATDIAG_SIZE,
+			      XmNheight, SATDIAG_SIZE+24, XmNwidth, SATDIAG_SIZE,
 			      NULL);
     gcv.foreground = BlackPixelOfScreen(XtScreen(satellite_diagram));
     gc = XCreateGC(XtDisplay(satellite_diagram),
@@ -157,18 +164,17 @@ static void build_gui(Widget toplevel)
     /*@ -immediatetrans @*/
     XtSetArg(args[0], XmNorientation, XmHORIZONTAL);
     /*@ +immediatetrans @*/
-    rowColumn_11 = XtCreateManagedWidget("time", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_11 = XtCreateManagedWidget("time     ", xmRowColumnWidgetClass, left, args, 1);
 
-    rowColumn_12 = XtCreateManagedWidget("latitude", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_12 = XtCreateManagedWidget("latitude ", xmRowColumnWidgetClass, left, args, 1);
     rowColumn_13 = XtCreateManagedWidget("longitude", xmRowColumnWidgetClass, left, args, 1);
-    rowColumn_14 = XtCreateManagedWidget("altitude", xmRowColumnWidgetClass, left, args, 1);
-    rowColumn_15 = XtCreateManagedWidget("speed", xmRowColumnWidgetClass, left, args, 1);
-    rowColumn_16 = XtCreateManagedWidget("track", xmRowColumnWidgetClass, left, args, 1);
-    rowColumn_17 = XtCreateManagedWidget("eph", xmRowColumnWidgetClass, left, args, 1);
-    rowColumn_18 = XtCreateManagedWidget("epv", xmRowColumnWidgetClass, left, args, 1);
-    rowColumn_19 = XtCreateManagedWidget("climb", xmRowColumnWidgetClass, left, args, 1);
-    rowColumn_20 = XtCreateManagedWidget("status", xmRowColumnWidgetClass, left, args, 1);
-    rowColumn_21 = XtCreateManagedWidget("quit", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_14 = XtCreateManagedWidget("altitude ", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_15 = XtCreateManagedWidget("speed    ", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_16 = XtCreateManagedWidget("track    ", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_17 = XtCreateManagedWidget("eph      ", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_18 = XtCreateManagedWidget("epv      ", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_19 = XtCreateManagedWidget("climb    ", xmRowColumnWidgetClass, left, args, 1);
+    rowColumn_20 = XtCreateManagedWidget("status   ", xmRowColumnWidgetClass, right, args, 1);
 
     label_1 = XtCreateManagedWidget("Time     ", xmLabelWidgetClass, rowColumn_11, args, 0);
     label_2 = XtCreateManagedWidget("Latitude ", xmLabelWidgetClass, rowColumn_12, args, 0);
@@ -210,12 +216,14 @@ static void build_gui(Widget toplevel)
     text_10 = XtCreateManagedWidget("text_10", xmTextFieldWidgetClass,
 				   rowColumn_20, args, 6);
 
-    quitbutton = XtCreateManagedWidget("Quit",
-			 xmPushButtonWidgetClass, rowColumn_21, args, 0);
+#ifdef EXPLICIT_QUIT
+    quitbutton = XtCreateManagedWidget("            Quit            ",
+			 xmPushButtonWidgetClass, rowColumn_20, args, 0);
     /*@i@*/XtAddCallback(quitbutton, XmNactivateCallback, (XtPointer)quit_cb, NULL);
+#endif /* EXPLICIT_QUIT */
 
     XtRealizeWidget(toplevel);
-    delw = XmInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW", False);
+    delw = XmInternAtom(XtDisplay(toplevel),"WM_DELETE_WINDOW",(Boolean)False);
     /*@ -nullpass @*//* splint 3.1.1 lacks annotated prototype... */
     /*@i@*/XmAddWMProtocolCallback(toplevel, delw,
 			    (XtCallbackProc)quit_cb, (XtPointer)NULL);
@@ -244,6 +252,7 @@ static void handle_time_out(XtPointer client_data UNUSED,
 static struct gps_data_t *gpsdata;
 static time_t timer;	/* time of last state change */
 static int state = 0;	/* or MODE_NO_FIX=1, MODE_2D=2, MODE_3D=3 */
+static int smoothing = 4;	/* # of transmitted sentences to smooth across */
 static XtAppContext app;
 static XtIntervalId timeout;
 static enum deg_str_type deg_type = deg_dd;
@@ -262,8 +271,9 @@ static void update_panel(struct gps_data_t *gpsdata,
 			 size_t len UNUSED, int level UNUSED)
 /* runs on each sentence */
 {
+    static int lfok = 0;
     unsigned int i;
-    int newstate;
+    int newstate, newtxt;
     XmString string[MAXCHANNELS+1];
     char s[128], *latlon, *sp;
 
@@ -271,9 +281,9 @@ static void update_panel(struct gps_data_t *gpsdata,
 	while (isspace(*(sp = message + strlen(message) - 1)))
 	    *sp = '\0';
     XmTextFieldSetString(status, message);
-    string[0] = XmStringCreateSimple("PRN:   Elev:  Azim:  SNR:  Used:");
     /* This is for the satellite status display */
     if (gpsdata->satellites) {
+	string[0] = XmStringCreateSimple("PRN:   Elev:  Azim:  SNR:  Used:");
 	for (i = 0; i < MAXCHANNELS; i++) {
 	    if (i < (unsigned int)gpsdata->satellites) {
 		(void)snprintf(s, sizeof(s),  
@@ -290,159 +300,176 @@ static void update_panel(struct gps_data_t *gpsdata,
 	    XmStringFree(string[i]);
     }
     /* here are the value fields */
-    if (isnan(gpsdata->fix.time)==0)
-	(void)unix_to_iso8601(gpsdata->fix.time, s, (int)sizeof(s));
-    else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_1, s);
-    if (gpsdata->fix.mode >= MODE_2D) {
-        latlon = deg_to_str(deg_type,  fabs(gpsdata->fix.latitude));
-	(void)snprintf(s, sizeof(s), "%s %c", latlon, (gpsdata->fix.latitude < 0) ? 'S' : 'N');
-    } else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_2, s);
-    if (gpsdata->fix.mode >= MODE_2D) {
-        latlon = deg_to_str(deg_type,  fabs(gpsdata->fix.longitude));
-	(void)snprintf(s, sizeof(s), "%s %c", latlon, (gpsdata->fix.longitude < 0) ? 'W' : 'E');
-    } else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_3, s);
-    if (gpsdata->fix.mode == MODE_3D)
-	(void)snprintf(s, sizeof(s), "%f %s",gpsdata->fix.altitude*altunits->factor, altunits->legend);
-    else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_4, s);
-    if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0)
-	(void)snprintf(s, sizeof(s), "%f %s", gpsdata->fix.speed*speedunits->factor, speedunits->legend);
-    else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_5, s);
-    if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0)
-	(void)snprintf(s, sizeof(s), "%f degrees", gpsdata->fix.track);
-    else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_6, s);
-    if (isnan(gpsdata->fix.eph)==0)
-	(void)snprintf(s, sizeof(s), "%f %s", gpsdata->fix.eph * altunits->factor, altunits->legend);
-    else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_7, s);
-    if (isnan(gpsdata->fix.epv)==0)
-	(void)snprintf(s, sizeof(s), "%f %s", gpsdata->fix.epv * altunits->factor, altunits->legend);
-    else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_8, s);
-    if (gpsdata->fix.mode == MODE_3D && isnan(gpsdata->fix.climb)==0)
-	(void)snprintf(s, sizeof(s), "%f %s/sec", gpsdata->fix.climb * altunits->factor, altunits->legend);
-    else
-	(void)strcpy(s, "n/a");
-    XmTextFieldSetString(text_9, s);
-
-    if (gpsdata->online == 0) {
-	newstate = 0;
-	(void)snprintf(s, sizeof(s), "OFFLINE");
-    } else {
-	newstate = gpsdata->fix.mode;
-	switch (gpsdata->fix.mode) {
-	case MODE_2D:
-	    (void)snprintf(s, sizeof(s), "2D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
-	    break;
-	case MODE_3D:
-	    (void)snprintf(s, sizeof(s), "3D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
-	    break;
-	default:
-	    (void)snprintf(s, sizeof(s), "NO FIX");
-	    break;
+    if (isnan(gpsdata->fix.time)==0) {
+	    (void)unix_to_iso8601(gpsdata->fix.time, s, (int)sizeof(s));
+	    newtxt = 1;
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
 	}
-    }
-    if (newstate != state) {
-	timer = time(NULL);
-	state = newstate;
-    }
-    (void)snprintf(s+strlen(s), sizeof(s)-strlen(s),
-		   " (%d secs)", (int) (time(NULL) - timer));
-    XmTextFieldSetString(text_10, s);
-    draw_graphics(gpsdata);
+	if (newtxt != 0) XmTextFieldSetString(text_1, s);
+	if (gpsdata->fix.mode >= MODE_2D) {
+	    latlon = deg_to_str(deg_type,  fabs(gpsdata->fix.latitude));
+	    newtxt = snprintf(s, sizeof(s), "%s %c", latlon, (gpsdata->fix.latitude < 0) ? 'S' : 'N');
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
+	}
+	if (newtxt != 0) XmTextFieldSetString(text_2, s);
+	if (gpsdata->fix.mode >= MODE_2D) {
+	    latlon = deg_to_str(deg_type,  fabs(gpsdata->fix.longitude));
+	    newtxt = snprintf(s, sizeof(s), "%s %c", latlon, (gpsdata->fix.longitude < 0) ? 'W' : 'E');
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
+	}
+	if (newtxt != 0) XmTextFieldSetString(text_3, s);
+	if (gpsdata->fix.mode == MODE_3D) {
+	    newtxt = snprintf(s, sizeof(s), "%f %s",gpsdata->fix.altitude*altunits->factor, altunits->legend);
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
+	}
+	if (newtxt != 0) XmTextFieldSetString(text_4, s);
+	if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0) {
+	    newtxt = snprintf(s, sizeof(s), "%f %s", gpsdata->fix.speed*speedunits->factor, speedunits->legend);
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
+	}
+	if (newtxt != 0) XmTextFieldSetString(text_5, s);
+	if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0) {
+	    newtxt = snprintf(s, sizeof(s), "%f degrees", gpsdata->fix.track);
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
+	}
+	if (newtxt != 0) XmTextFieldSetString(text_6, s);
+	if (isnan(gpsdata->fix.eph)==0) {
+	    newtxt = snprintf(s, sizeof(s), "%f %s", gpsdata->fix.eph * altunits->factor, altunits->legend);
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
+	}
+	if (newtxt != 0) XmTextFieldSetString(text_7, s);
+	if (isnan(gpsdata->fix.epv)==0) {
+	    newtxt = snprintf(s, sizeof(s), "%f %s", gpsdata->fix.epv * altunits->factor, altunits->legend);
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
+	}
+	if (newtxt != 0) XmTextFieldSetString(text_8, s);
+	if (gpsdata->fix.mode == MODE_3D && isnan(gpsdata->fix.climb)==0) {
+	    newtxt = snprintf(s, sizeof(s), "%f %s/sec", gpsdata->fix.climb * altunits->factor, altunits->legend);
+	} else {
+	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
+	}
+	if (newtxt != 0) XmTextFieldSetString(text_9, s);
 
-    XtRemoveTimeOut(timeout);
-    timeout = XtAppAddTimeOut(app, 2000, handle_time_out, NULL);
-}
-
-static char *get_resource(Widget w, char *name, char *default_value)
-{
-  XtResource xtr;
-  char *value = NULL;
-
-  /*@ -observertrans -statictrans -immediatetrans -compdestroy @*/
-  xtr.resource_name = name;
-  xtr.resource_class = "AnyClass";
-  xtr.resource_type = XmRString;
-  xtr.resource_size = (Cardinal)sizeof(String);
-  xtr.resource_offset = 0;
-  xtr.default_type = XmRImmediate;
-  xtr.default_addr = default_value;
-  XtGetApplicationResources(w, &value, &xtr, 1, NULL, 0);
-  if (value) return value;
-  /*@ +observertrans +statictrans +immediatetrans +compdestroy @*/
-  /*@i@*/return default_value;
-}
-
-/*@ -mustfreefresh @*/
-int main(int argc, char *argv[])
-{
-    int option;
-    char *arg = NULL, *colon1, *colon2, *device = NULL, *server = NULL, *port = DEFAULT_GPSD_PORT;
-    char *su, *au;
-    char *err_str = NULL;
-
-    /*@ -onlytrans */
-    toplevel = XtVaAppInitialize(&app, "xgps", 
-			       options, XtNumber(options), 
-			       &argc,argv, fallback_resources,NULL);
-
-    /*@ +onlytrans */
-    su = get_resource(toplevel, "speedunits", "mph");
-    for (speedunits = speedtable; 
-	 speedunits < speedtable + sizeof(speedtable)/sizeof(speedtable[0]);
-	 speedunits++)
-	if (strcmp(speedunits->legend, su) == 0)
-	    goto speedunits_ok;
-    speedunits = speedtable;
-    (void)fprintf(stderr, "xgps: unknown speed unit, defaulting to %s\n", speedunits->legend);
-speedunits_ok:;
-
-    au = get_resource(toplevel, "altunits",   "feet");
-    for (altunits = alttable; 
-	 altunits < alttable + sizeof(alttable)/sizeof(alttable[0]);
-	 altunits++)
-	if (strcmp(altunits->legend, au) == 0)
-	    goto altunits_ok;
-    altunits = alttable;
-    (void)fprintf(stderr, "xgps: unknown altitude unit, defaulting to %s\n", altunits->legend);
-altunits_ok:;
-
-    while ((option = getopt(argc, argv, "hl:v")) != -1) {
-	switch (option) {
-	case 'v':
-	    (void)printf("xgps %s\n", VERSION);
-	    exit(0);
-	case 'l':
-	    switch ( optarg[0] ) {
-	    case 'd':
-		deg_type = deg_dd;
-		continue;
-	    case 'm':
-		deg_type = deg_ddmm;
-		continue;
-	    case 's':
-		deg_type = deg_ddmmss;
-		continue;
-            default:
-		(void)fprintf(stderr, "Unknown -l argument: %s\n", optarg);
-		/*@ -casebreak @*/
+	if (gpsdata->online == 0) {
+	    newstate = 0;
+	    (void)snprintf(s, sizeof(s), "OFFLINE");
+	    lfok = 0;
+	} else {
+	    newstate = gpsdata->fix.mode;
+	    switch (gpsdata->fix.mode) {
+	    case MODE_2D:
+		(void)snprintf(s, sizeof(s), "2D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
+		if (lfok>0) lfok--;
+		break;
+	    case MODE_3D:
+		(void)snprintf(s, sizeof(s), "3D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
+		lfok = smoothing;
+		break;
+	    default:
+		(void)snprintf(s, sizeof(s), "NO FIX");
+		lfok = 0;
+		break;
 	    }
+	}
+	if (newstate != state) {
+	    timer = time(NULL);
+	    state = newstate;
+	}
+	(void)snprintf(s+strlen(s), sizeof(s)-strlen(s),
+		       " (%d secs)", (int) (time(NULL) - timer));
+	XmTextFieldSetString(text_10, s);
+	draw_graphics(gpsdata);
+
+	XtRemoveTimeOut(timeout);
+	timeout = XtAppAddTimeOut(app, 2000, handle_time_out, NULL);
+    }
+
+    static char *get_resource(Widget w, char *name, char *default_value)
+    {
+      XtResource xtr;
+      char *value = NULL;
+
+      /*@ -observertrans -statictrans -immediatetrans -compdestroy @*/
+      xtr.resource_name = name;
+      xtr.resource_class = "AnyClass";
+      xtr.resource_type = XmRString;
+      xtr.resource_size = (Cardinal)sizeof(String);
+      xtr.resource_offset = 0;
+      xtr.default_type = XmRImmediate;
+      xtr.default_addr = default_value;
+      XtGetApplicationResources(w, &value, &xtr, 1, NULL, 0);
+      if (value) return value;
+      /*@ +observertrans +statictrans +immediatetrans +compdestroy @*/
+      /*@i@*/return default_value;
+    }
+
+    /*@ -mustfreefresh @*/
+    int main(int argc, char *argv[])
+    {
+	int option;
+	char *arg = NULL, *colon1, *colon2, *device = NULL, *server = NULL, *port = DEFAULT_GPSD_PORT;
+	char *su, *au;
+	char *err_str = NULL;
+
+	/*@ -onlytrans */
+	toplevel = XtVaAppInitialize(&app, "xgps", 
+				   options, XtNumber(options), 
+				   &argc,argv, fallback_resources,NULL);
+
+	/*@ +onlytrans */
+	su = get_resource(toplevel, "speedunits", "mph");
+	for (speedunits = speedtable; 
+	     speedunits < speedtable + sizeof(speedtable)/sizeof(speedtable[0]);
+	     speedunits++)
+	    if (strcmp(speedunits->legend, su) == 0)
+		goto speedunits_ok;
+	speedunits = speedtable;
+	(void)fprintf(stderr, "xgps: unknown speed unit, defaulting to %s\n", speedunits->legend);
+    speedunits_ok:;
+
+	au = get_resource(toplevel, "altunits",   "feet");
+	for (altunits = alttable; 
+	     altunits < alttable + sizeof(alttable)/sizeof(alttable[0]);
+	     altunits++)
+	    if (strcmp(altunits->legend, au) == 0)
+		goto altunits_ok;
+	altunits = alttable;
+	(void)fprintf(stderr, "xgps: unknown altitude unit, defaulting to %s\n", altunits->legend);
+    altunits_ok:;
+
+	while ((option = getopt(argc, argv, "hl:s:V")) != -1) {
+	    switch (option) {
+	    case 'V':
+		(void)printf("xgps %s\n", VERSION);
+		exit(0);
+	    case 'l':
+		switch ( optarg[0] ) {
+		case 'd':
+		    deg_type = deg_dd;
+		    continue;
+		case 'm':
+		    deg_type = deg_ddmm;
+		    continue;
+		case 's':
+		    deg_type = deg_ddmmss;
+		    continue;
+		default:
+		    (void)fprintf(stderr, "Unknown -l argument: %s\n", optarg);
+		    /*@ -casebreak @*/
+		}
+	case 's':
+	    smoothing = atoi(optarg);
+	    break;
 	case 'h': default:
-	    (void)fputs("usage:  xgps [-hv] [-speedunits {mph,kph,knots}] [-altunits {ft,meters}] [-l {d|m|s}] [server[:port:[device]]]\n", stderr);
+	    (void)fputs("usage:  xgps [-hV] [-speedunits {mph,kph,knots}] [-altunits {ft,meters}] [-l {d|m|s}] [-s smoothing] [server[:port:[device]]]\n", stderr);
 	    exit(1);
 	}
     }
