@@ -7,6 +7,8 @@
 #include <unistd.h>
 #include <math.h>
 #include "gpsd.h"
+
+#define LITTLE_ENDIAN_PROTOCOL
 #include "bits.h"
 
 #ifdef ZODIAC_ENABLE
@@ -82,12 +84,12 @@ static bool zodiac_speed_switch(struct gps_device_t *session, speed_t speed)
 {
     unsigned short data[15];
 
-    if (session->zodiac.sn++ > 32767)
-	session->zodiac.sn = 0;
+    if (session->driver.zodiac.sn++ > 32767)
+	session->driver.zodiac.sn = 0;
       
     memset(data, 0, sizeof(data));
     /* data is the part of the message starting at word 6 */
-    data[0] = session->zodiac.sn;	/* sequence number */
+    data[0] = session->driver.zodiac.sn;	/* sequence number */
     data[1] = 1;			/* port 1 data valid */
     data[2] = 1;			/* port 1 character width (8 bits) */
     data[3] = 0;			/* port 1 stop bits (1 stopbit) */
@@ -106,11 +108,11 @@ static void send_rtcm(struct gps_device_t *session,
     unsigned short data[34];
     int n = 1 + (int)(rtcmbytes/2 + rtcmbytes%2);
 
-    if (session->zodiac.sn++ > 32767)
-	session->zodiac.sn = 0;
+    if (session->driver.zodiac.sn++ > 32767)
+	session->driver.zodiac.sn = 0;
 
     memset(data, 0, sizeof(data));
-    data[0] = session->zodiac.sn;		/* sequence number */
+    data[0] = session->driver.zodiac.sn;		/* sequence number */
     memcpy(&data[1], rtcmbuf, rtcmbytes);
     data[n] = zodiac_checksum(data, n);
 
@@ -133,6 +135,8 @@ static ssize_t zodiac_send_rtcm(struct gps_device_t *session,
 
 static gps_mask_t handle1000(struct gps_device_t *session)
 {
+    double subseconds;
+    struct tm unpacked_date;
     /* ticks                      = getlong(6); */
     /* sequence                   = getword(8); */
     /* measurement_sequence       = getword(9); */
@@ -150,15 +154,17 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     /* gps_week                      = getword(14); */
     /* gps_seconds                   = getlong(15); */
     /* gps_nanoseconds               = getlong(17); */
-    session->gpsdata.nmea_date.tm_mday = (int)getword(19);
-    session->gpsdata.nmea_date.tm_mon = (int)getword(20) - 1;
-    session->gpsdata.nmea_date.tm_year = (int)getword(21) - 1900;
-    session->gpsdata.nmea_date.tm_hour = (int)getword(22);
-    session->gpsdata.nmea_date.tm_min = (int)getword(23);
-    session->gpsdata.nmea_date.tm_sec = (int)getword(24);
-    session->gpsdata.subseconds = (int)getlong(25) / 1e9;
+    unpacked_date.tm_mday = (int)getword(19);
+    unpacked_date.tm_mon = (int)getword(20) - 1;
+    unpacked_date.tm_year = (int)getword(21) - 1900;
+    unpacked_date.tm_hour = (int)getword(22);
+    unpacked_date.tm_min = (int)getword(23);
+    unpacked_date.tm_sec = (int)getword(24);
+    subseconds = (int)getlong(25) / 1e9;
+    /*@ -compdef */
     session->gpsdata.newdata.time = session->gpsdata.sentence_time =
-	(double)mkgmtime(&session->gpsdata.nmea_date) + session->gpsdata.subseconds;
+	(double)mkgmtime(&unpacked_date) + subseconds;
+    /*@ +compdef */
 #ifdef NTPSHM_ENABLE
     if (session->gpsdata.newdata.mode > MODE_NO_FIX)
 	(void)ntpshm_put(session, session->gpsdata.newdata.time + 1.1);
@@ -180,10 +186,11 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     session->mag_var               = ((short)getword(37)) * RAD_2_DEG * 1e-4;
     session->gpsdata.newdata.climb     = ((short)getword(38)) * 1e-2;
     /* map_datum                   = getword(39); */
-    session->gpsdata.newdata.eph       = (int)getlong(40) * 1e-2;
-    session->gpsdata.newdata.epv       = (int)getlong(42) * 1e-2;
-    session->gpsdata.newdata.ept       = (int)getlong(44) * 1e-2;
-    session->gpsdata.newdata.eps       = (int)getword(46) * 1e-2;
+    /* manual says these are 1-sigma */
+    session->gpsdata.newdata.eph       = (int)getlong(40) * 1e-2 * GPSD_CONFIDENCE;
+    session->gpsdata.newdata.epv       = (int)getlong(42) * 1e-2 * GPSD_CONFIDENCE;
+    session->gpsdata.newdata.ept       = (int)getlong(44) * 1e-2 * GPSD_CONFIDENCE;
+    session->gpsdata.newdata.eps       = (int)getword(46) * 1e-2 * GPSD_CONFIDENCE;
     /* clock_bias                  = (int)getlong(47) * 1e-2; */
     /* clock_bias_sd               = (int)getlong(49) * 1e-2; */
     /* clock_drift                 = (int)getlong(51) * 1e-2; */
@@ -213,7 +220,7 @@ static gps_mask_t handle1000(struct gps_device_t *session)
 #endif
 
     session->gpsdata.sentence_length = 55;
-    return TIME_SET|LATLON_SET|ALTITUDE_SET|CLIMB_SET|SPEED_SET|TRACK_SET|STATUS_SET|MODE_SET|HERR_SET|VERR_SET|SPEEDERR_SET|CYCLE_START_SET;
+    return TIME_SET|LATLON_SET|ALTITUDE_SET|CLIMB_SET|SPEED_SET|TRACK_SET|STATUS_SET|MODE_SET|CYCLE_START_SET; /* |HERR_SET|VERR_SET|SPEEDERR_SET */
 }
 
 static gps_mask_t handle1002(struct gps_device_t *session)
@@ -230,8 +237,8 @@ static gps_mask_t handle1002(struct gps_device_t *session)
     /* gps_nanoseconds            = getlong(13); */
     for (i = 0; i < ZODIAC_CHANNELS; i++) {
 	/*@ -type @*/ 
-	session->zodiac.Zv[i] = status = (int)getword(15 + (3 * i));
-	session->zodiac.Zs[i] = prn = (int)getword(16 + (3 * i));
+	session->driver.zodiac.Zv[i] = status = (int)getword(15 + (3 * i));
+	session->driver.zodiac.Zs[i] = prn = (int)getword(16 + (3 * i));
 	/*@ +type @*/ 
 #if 0
 	gpsd_report(1, "Sat%02d:\n", i);
@@ -251,7 +258,7 @@ static gps_mask_t handle1002(struct gps_device_t *session)
 	    break;
 	}
     }
-    return SATELLITE_SET;
+    return SATELLITE_SET | USED_SET;
 }
 
 static gps_mask_t handle1003(struct gps_device_t *session)
@@ -295,7 +302,7 @@ static void handle1005(struct gps_device_t *session UNUSED)
 #if 0
     int i;
 
-    gpsd_report(1, "Packet: %d\n", session->zodiac.sn);
+    gpsd_report(1, "Packet: %d\n", session->driver.zodiac.sn);
     gpsd_report(1, "Station bad: %d\n", (getword(9) & 1) ? 1 : 0);
     gpsd_report(1, "User disabled: %d\n", (getword(9) & 2) ? 1 : 0);
     gpsd_report(1, "Station ID: %d\n", getword(10));
@@ -380,17 +387,20 @@ static gps_mask_t zodiac_analyze(struct gps_device_t *session)
 /* this is everything we export */
 struct gps_type_t zodiac_binary =
 {
-    "Zodiac binary",	/* full name of type */
-    NULL,		/* no probe */
-    NULL,		/* only switched to by some other driver */
-    NULL,		/* no initialization */
-    packet_get,		/* how to get a packet */
-    zodiac_analyze,	/* read and parse message packets */
-    zodiac_send_rtcm,	/* send DGPS correction */
-    zodiac_speed_switch,/* we can change baud rate */
-    NULL,		/* no mode switcher */
-    NULL,		/* caller needs to supply a close hook */
-    1,			/* updates every second */
+    .typename       = "Zodiac binary",	/* full name of type */
+    .trigger        = NULL,		/* no trigger */
+    .channels       = 12,		/* consumer-grade GPS */
+    .probe          = NULL,		/* no probe */
+    .initializer    = NULL,		/* no initialization */
+    .get_packet     = packet_get,	/* use the generic packet getter */
+    .parse_packet   = zodiac_analyze,	/* parse message packets */
+    .rtcm_writer    = zodiac_send_rtcm,	/* send DGPS correction */
+    .speed_switcher = zodiac_speed_switch,/* we can change baud rate */
+    .mode_switcher  = NULL,		/* no mode switcher */
+    .rate_switcher  = NULL,		/* no sample-rate switcher */
+    .cycle_chars    = -1,		/* not relevant, no rate switch */
+    .wrapup         = NULL,		/* caller might supply a close hook */
+    .cycle          = 1,		/* updates every second */
 };
 
 #endif /* ZODIAC_ENABLE */
